==> af-common-plugins-2.14.7-sources.jar: com/af/v4/system/common/plugins/concurrent/BatchTaskProcessor.java <==
package com.af.v4.system.common.plugins.concurrent;

import com.af.v4.system.common.core.context.GlobalVisualThreadContext;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.stream.Stream;

/**
 * BatchTaskProcessor
 * <p>
 * 一个通用的批量任务处理器，支持将大规模任务列表分块并发处理。
 *
 * <p>该类提供以下功能：
 * <ul>
 *   <li>通过动态计算分块大小，将任务列表拆分为多个子列表</li>
 *   <li>通过虚拟线程池实现高效并发处理</li>
 *   <li>使用者只需专注于单个任务的处理逻辑</li>
 *   <li>自动统计成功和失败的任务数量</li>
 *   <li>支持任务预处理及处理完成后的回调</li>
 *   <li>确保所有任务完成后再返回，适合对结果一致性有要求的场景</li>
 * </ul>
 *
 * @param <T> 要处理的任务类型
 */
public class BatchTaskProcessor<T> {

    private static final Logger LOGGER = LoggerFactory.getLogger(BatchTaskProcessor.class);

    /**
     * 动态计算分块大小，根据任务数量和服务器核心数动态调整。
     * 提供两种分块计算策略：针对 IO 密集型任务和 CPU 密集型任务。
     * <p>
     * 计算公式：
     * <pre>
     * - IO 密集型：chunkSize = listSize / (availableCores * 2 + 1)
     * - CPU 密集型：chunkSize = listSize /（availableCores + 1）
     * </pre>
     * <p>
     * 以下是任务数量与预期块数量和每块任务数（核心数 4）：
     * <table>
     *   <thead>
     *     <tr>
     *       <th>任务列表大小</th>
     *       <th>每块任务数量 (IO 密集型)</th>
     *       <th>每块任务数量 (CPU 密集型)</th>
     *     </tr>
     *   </thead>
     *   <tbody>
     *     <tr><td>500</td><td>55</td><td>100</td></tr>
     *     <tr><td>1000</td><td>111</td><td>200</td></tr>
     *     <tr><td>2000</td><td>222</td><td>400</td></tr>
     *     <tr><td>5000</td><td>555</td><td>1000</td></tr>
     *     <tr><td>10000</td><td>1111</td><td>2000</td></tr>
     *   </tbody>
     * </table>
     *
     * @param listSize 任务列表的大小
     * @param taskType 任务类型（IO 密集型或 CPU 密集型）
     * @return 动态计算后的分块大小
     */
    protected int calculateChunkSize(int listSize, TaskType taskType) {
        int result;
        int availableCores = 4;
        if (taskType == TaskType.CPU_INTENSIVE) {
            // CPU 密集型任务
            result = Math.max(1, listSize / (availableCores + 1));
        } else {
            // IO 密集型任务
            result = Math.max(1, listSize / (availableCores * 2 + 1));
        }
        return result;
    }

    /**
     * 按分块并发处理任务列表。
     *
     * <p>该方法将任务列表分割为多个任务块，使用虚拟线程池并发执行，
     * 并通过传入的处理逻辑对分块中的每个任务执行操作。任务完成后会打印成功和失败的任务统计信息。
     *
     * @param items           待处理的任务列表，不可为空
     * @param taskType        任务类型（IO 密集型或 CPU 密集型）
     * @param taskHandler     单个任务的处理逻辑，接受一个任务作为输入
     * @param preProcessor    （可选）在分块之前对任务列表进行预处理，返回处理后的任务列表
     * @param successCallback （可选）每个成功处理的任务的回调
     * @param failureCallback （可选）每个失败处理的任务的回调
     * @return 统计结果，包含成功和失败的任务数量、分块数量等
     */
    public <V> JSONObject process(
            List<T> items,
            TaskType taskType,
            Function<T, V> taskHandler,
            Function<List<T>, List<T>> preProcessor,
            BiConsumer<T, V> successCallback,
            BiConsumer<T, Exception> failureCallback
    ) {

        // 依据任务列表大小和 CPU 核心数动态计算 chunkSize
        int chunkSize = calculateChunkSize(items.size(), taskType);

        // 可选的任务预处理
        List<T> processedItems = preProcessor != null ? preProcessor.apply(items) : items;

        // 分割任务列表为多个子列表
        List<List<T>> chunks = splitList(processedItems, chunkSize);
        List<CompletableFuture<Void>> futures = new ArrayList<>();

        JSONObject result = new JSONObject();
        AtomicInteger successCount = new AtomicInteger(0);
        AtomicInteger failureCount = new AtomicInteger(0);

        // 为每个分块创建异步任务
        for (List<T> chunk : chunks) {
            CompletableFuture<Void> future = GlobalVisualThreadContext.runAsync(() -> {
                for (T item : chunk) {
                    try {
                        V resultValue = taskHandler.apply(item);
                        successCount.incrementAndGet(); // 成功计数自增
                        if (successCallback != null) {
                            successCallback.accept(item, resultValue);
                        }
                    } catch (Exception e) {
                        failureCount.incrementAndGet(); // 失败计数自增
                        if (failureCallback != null) {
                            failureCallback.accept(item, e);
                        }
                        LOGGER.error("Error processing task: {}", item, e);
                    }
                }
            });
            futures.add(future);
        }

        // 等待所有任务完成
        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).join();

        // 统计结果
        result.put("totalItems", processedItems.size());
        result.put("chunkSize", chunkSize);
        result.put("totalChunks", chunks.size());
        result.put("successCount", successCount.get());
        result.put("failureCount", failureCount.get());

        LOGGER.info("Batch task processing completed: {}", result);
        return result;
    }


    public <V> JSONObject process(
            List<T> items,
            Function<T, V> taskHandler
    ) {
        return process(items, TaskType.IO_INTENSIVE, taskHandler, null, null, null);
    }

    /**
     * 将列表按指定大小分割为多个子列表。
     *
     * <p>如果列表长度不能被分块大小整除，则最后一个子列表的大小可能小于分块大小。
     *
     * @param list      待分割的列表，不可为空
     * @param chunkSize 每个分块的大小，必须大于 0
     * @return 按分块大小拆分后的子列表集合
     */
    protected List<List<T>> splitList(List<T> list, int chunkSize) {
        List<List<T>> chunks = new ArrayList<>();
        int length = list.size();
        for (int i = 0; i < length; i += chunkSize) {
            int end = Math.min(i + chunkSize, length);
            chunks.add(list.subList(i, end));
        }
        return chunks;
    }

    /**
     * 任务类型枚举
     */
    public enum TaskType {
        IO_INTENSIVE("io"), // IO 密集型任务
        CPU_INTENSIVE("cpu"); // CPU 密集型任务

        final String value;

        TaskType(String value) {
            this.value = value;
        }

        public static TaskType toType(String value) {
            return Stream.of(TaskType.values())
                    .filter(p -> p.value.equals(value))
                    .findAny()
                    .orElse(null);
        }

        public static boolean is(String value) {
            return toType(value) != null;
        }

        public String getValue() {
            return value;
        }
    }
}

==> af-common-plugins-2.14.41-sources.jar: com/af/v4/system/common/plugins/concurrent/BatchTaskProcessor.java <==
package com.af.v4.system.common.plugins.concurrent;

import com.af.v4.system.common.core.context.GlobalVisualThreadContext;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.stream.Stream;

/**
 * BatchTaskProcessor
 * <p>
 * 一个通用的批量任务处理器，支持将大规模任务列表分块并发处理。
 *
 * <p>该类提供以下功能：
 * <ul>
 *   <li>通过动态计算分块大小，将任务列表拆分为多个子列表</li>
 *   <li>通过虚拟线程池实现高效并发处理</li>
 *   <li>使用者只需专注于单个任务的处理逻辑</li>
 *   <li>自动统计成功和失败的任务数量</li>
 *   <li>支持任务预处理及处理完成后的回调</li>
 *   <li>确保所有任务完成后再返回，适合对结果一致性有要求的场景</li>
 * </ul>
 *
 * @param <T> 要处理的任务类型
 */
public class BatchTaskProcessor<T> {

    private static final Logger LOGGER = LoggerFactory.getLogger(BatchTaskProcessor.class);

    /**
     * 动态计算分块大小，根据任务数量和服务器核心数动态调整。
     * 提供两种分块计算策略：针对 IO 密集型任务和 CPU 密集型任务。
     * <p>
     * 计算公式：
     * <pre>
     * - IO 密集型：chunkSize = listSize / (availableCores * 2 + 1)
     * - CPU 密集型：chunkSize = listSize /（availableCores + 1）
     * </pre>
     * <p>
     * 以下是任务数量与预期块数量和每块任务数（核心数 4）：
     * <table>
     *   <thead>
     *     <tr>
     *       <th>任务列表大小</th>
     *       <th>每块任务数量 (IO 密集型)</th>
     *       <th>每块任务数量 (CPU 密集型)</th>
     *     </tr>
     *   </thead>
     *   <tbody>
     *     <tr><td>500</td><td>55</td><td>100</td></tr>
     *     <tr><td>1000</td><td>111</td><td>200</td></tr>
     *     <tr><td>2000</td><td>222</td><td>400</td></tr>
     *     <tr><td>5000</td><td>555</td><td>1000</td></tr>
     *     <tr><td>10000</td><td>1111</td><td>2000</td></tr>
     *   </tbody>
     * </table>
     *
     * @param listSize 任务列表的大小
     * @param taskType 任务类型（IO 密集型或 CPU 密集型）
     * @return 动态计算后的分块大小
     */
    protected int calculateChunkSize(int listSize, TaskType taskType) {
        int result;
        int availableCores = Runtime.getRuntime().availableProcessors();
        if (taskType == TaskType.CPU_INTENSIVE) {
            // CPU 密集型任务
            result = Math.max(1, listSize / (availableCores + 1));
        } else {
            // IO 密集型任务
            result = Math.max(1, listSize / (availableCores * 2 + 1));
        }
        return result;
    }

    /**
     * 按分块并发处理任务列表。
     *
     * <p>该方法将任务列表分割为多个任务块，使用虚拟线程池并发执行，
     * 并通过传入的处理逻辑对分块中的每个任务执行操作。任务完成后会打印成功和失败的任务统计信息。
     *
     * @param items           待处理的任务列表，不可为空
     * @param taskType        任务类型（IO 密集型或 CPU 密集型）
     * @param taskHandler     单个任务的处理逻辑，接受一个任务作为输入
     * @param preProcessor    （可选）在分块之前对任务列表进行预处理，返回处理后的任务列表
     * @param successCallback （可选）每个成功处理的任务的回调
     * @param failureCallback （可选）每个失败处理的任务的回调
     * @return 统计结果，包含成功和失败的任务数量、分块数量等
     */
    public <V> JSONObject process(
            List<T> items,
            TaskType taskType,
            Function<T, V> taskHandler,
            Function<List<T>, List<T>> preProcessor,
            BiConsumer<T, V> successCallback,
            BiConsumer<T, Exception> failureCallback
    ) {

        // 依据任务列表大小和 CPU 核心数动态计算 chunkSize
        int chunkSize = calculateChunkSize(items.size(), taskType);

        // 可选的任务预处理
        List<T> processedItems = preProcessor != null ? preProcessor.apply(items) : items;

        // 分割任务列表为多个子列表
        List<List<T>> chunks = splitList(processedItems, chunkSize);
        List<CompletableFuture<Void>> futures = new ArrayList<>();

        JSONObject result = new JSONObject();
        AtomicInteger successCount = new AtomicInteger(0);
        AtomicInteger failureCount = new AtomicInteger(0);

        // 为每个分块创建异步任务
        for (List<T> chunk : chunks) {
            CompletableFuture<Void> future = GlobalVisualThreadContext.runAsync(() -> {
                for (T item : chunk) {
                    try {
                        V resultValue = taskHandler.apply(item);
                        successCount.incrementAndGet(); // 成功计数自增
                        if (successCallback != null) {
                            successCallback.accept(item, resultValue);
                        }
                    } catch (Exception e) {
                        failureCount.incrementAndGet(); // 失败计数自增
                        if (failureCallback != null) {
                            failureCallback.accept(item, e);
                        }
                        LOGGER.error("Error processing task: {}", item, e);
                    }
                }
            });
            futures.add(future);
        }

        // 等待所有任务完成
        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).join();

        // 统计结果
        result.put("totalItems", processedItems.size());
        result.put("chunkSize", chunkSize);
        result.put("totalChunks", chunks.size());
        result.put("successCount", successCount.get());
        result.put("failureCount", failureCount.get());

        LOGGER.info("Batch task processing completed: {}", result);
        return result;
    }


    public <V> JSONObject process(
            List<T> items,
            Function<T, V> taskHandler
    ) {
        return process(items, TaskType.IO_INTENSIVE, taskHandler, null, null, null);
    }

    /**
     * 将列表按指定大小分割为多个子列表。
     *
     * <p>如果列表长度不能被分块大小整除，则最后一个子列表的大小可能小于分块大小。
     *
     * @param list      待分割的列表，不可为空
     * @param chunkSize 每个分块的大小，必须大于 0
     * @return 按分块大小拆分后的子列表集合
     */
    protected List<List<T>> splitList(List<T> list, int chunkSize) {
        List<List<T>> chunks = new ArrayList<>();
        int length = list.size();
        for (int i = 0; i < length; i += chunkSize) {
            int end = Math.min(i + chunkSize, length);
            chunks.add(list.subList(i, end));
        }
        return chunks;
    }

    /**
     * 任务类型枚举
     */
    public enum TaskType {
        IO_INTENSIVE("io"), // IO 密集型任务
        CPU_INTENSIVE("cpu"); // CPU 密集型任务

        final String value;

        TaskType(String value) {
            this.value = value;
        }

        public static TaskType toType(String value) {
            return Stream.of(TaskType.values())
                    .filter(p -> p.value.equals(value))
                    .findAny()
                    .orElse(null);
        }

        public static boolean is(String value) {
            return toType(value) != null;
        }

        public String getValue() {
            return value;
        }
    }
}
